package info.textgrid.middleware;

import info.textgrid.middleware.oaipmh.OAIPMHType;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.core.MediaType;
import javax.xml.bind.JAXBElement;

/**
 * Interface to the get Request function
 * 
 * @author Maximilian Brodhun, SUB
 * @version 1.0
 * @since 15.01.2014
 */

public interface OAIPMHProducer {

	/**
	 * 
	 * @param verb
	 *            The parameter verb is a required parameter to all requests.
	 *            Depending on it the request will be proceed. Possible values
	 *            are: GetRecord, Identify, ListIdentifiers,
	 *            ListMetadataFormats, ListRecords and ListSets In case of other
	 *            verbs an error response will be send back
	 * 
	 * @param identifier
	 *            This value indicates if the request will be proceed just for a
	 *            single identifier of the repository
	 * @param metadataPrefix
	 *            To limit the response just for a special metadata format this
	 *            value can be used. The prefix "oai_dc" has to be supported by
	 *            all repositories
	 * @param set
	 *            To represent the architecture of the repository it is possible
	 *            to make sets which work like categories. At the moment
	 *            textgrid does not support sets
	 * @param from
	 *            Start value to filter the response for a specific interval
	 * @param until
	 *            End value to filter the response for a specific interval
	 * @param resumptionToken
	 *            Indicates how many value will be send back in the response
	 *
	 */
    @POST
	@GET
	@Path("/")
	@Produces(MediaType.TEXT_XML)
	JAXBElement<OAIPMHType> getRequest(
			@QueryParam("verb") String verb,
			@QueryParam("identifier") @DefaultValue("") String identifier,
			@QueryParam("metadataPrefix") @DefaultValue("") String metadataPrefix,
			@QueryParam("set") @DefaultValue("") String set,
			@QueryParam("from") @DefaultValue("") String from,
			@QueryParam("until") @DefaultValue("") String until,
			@QueryParam("resumptionToken") @DefaultValue("") String resumptionToken);

	/**
	 * @return
	 */
	@GET
	@Path("/version")
	@Produces(MediaType.TEXT_PLAIN)
	String getVersion();

}
